##############################################################################
#   Title: draft_accountants.R
# Purpose: Takes collapsed output from inside analysis and produces graphs in
#          external environment.
#   Owner: EZ
# Created: 2020-04-06
##############################################################################

init <- function() {
  
  # Configure environment.
  MAINROOT <<- path.expand(".")
  FIG_OUTDIR <<- file.path(MAINROOT, "drafts/accountants/12pub/figures")
  TAB_OUTDIR <<- file.path(MAINROOT, "drafts/accountants/12pub/tables")
  DATADIR <<- file.path(MAINROOT, "PATH/TO/DATA")
  
  # Add packages
  require(data.table)
  require(ggplot2)
  require(ggthemes)
  require(RSQLite)
  
  # Graph parameters.
  WIDTH <<- 6.5
  HEIGHT <<- 4.25
}

# Figure 1
graph_losses <- function() { # %<
  
  data = data.table(read.csv(file.path(DATADIR,
                                       "loss_history.csv.xls")))
  
  reshaped <- rbind(data[, list(tax_yr, v = actual_refunds,
                                Group = "Actual Refunds")],
                    data[, list(tax_yr, v = sim_refunds, 
                                Group = "Simulated Refunds")])
  
  ggsave(filename = file.path(FIG_OUTDIR, "nol_aggs.pdf"),
         width = WIDTH, height = HEIGHT, plot =
           ggplot() +
           theme_few() +
           labs(x = "", 
                y = "Aggregate Carryback Refunds ($B)") +
           geom_bar(data = reshaped, aes(x = as.character(tax_yr), y = v*1e-9, fill = Group), 
                    position = "dodge",
                    stat = "identity") + 
           scale_fill_manual(values = c(rgb(102, 102, 255, max = 255), 
                                      rgb(204, 204, 255, max = 255))) +
           theme(legend.position = "bottom", legend.title = element_blank())
  )
  
  data$claim_share <- with(data, claimers_big/eligibles_big)
  data$claim_share_agg <- with(data, actual_refunds/sim_refunds)
  
  reshaped <- rbind(data[, list(tax_yr, v = claim_share,
                                Group = "Number Claimed")],
                    data[, list(tax_yr, v = claim_share_agg,
                                Group = "Dollars Claimed")])
  
  ggsave(filename = file.path(FIG_OUTDIR, "nol_shares.pdf"),
         width = WIDTH, height = HEIGHT, plot =
           ggplot() +
           theme_few() +
           labs(x = "", 
                y = "Share of Total (Percent)") +
           geom_bar(data = reshaped, aes(x = as.character(tax_yr), y = v, 
                                       fill = Group), 
                    position = "dodge",
                    stat = "identity") + 
           scale_fill_manual(values = c(rgb(102, 102, 255, max = 255), 
                                      rgb(204, 204, 255, max = 255))) +
           scale_y_continuous(limits = c(0,1)) +
           theme(legend.position = "bottom", legend.title = element_blank())
  )
} # %>

# Figure 2a, 2b
graph_npv_density <- function() { # %<
  
  data = data.table(read.csv(file.path(DATADIR,
                                       "npv_sim_20191103.csv")))
  
  data$share = data$density
  data = data[bins != "."]
  data$midpoints = as.numeric(as.character(data$bins)) + 125
  
  ggsave(filename = file.path(FIG_OUTDIR, "npv_sim.pdf"),
         width = 5, height = HEIGHT, plot =
           ggplot() +
           theme_few() +
           labs(x = "NPV Difference Carryback Minus Carryforward ($2013)", 
                y = "Density") +
           geom_bar(data = data, aes(x = reorder(as.character(midpoints), midpoints), 
                                   y = share), 
                    stat = "identity", fill = rgb(102, 102, 255, max = 255)) +
           geom_vline(data = data, aes(x = reorder(as.character(midpoints),
                                               midpoints)), xintercept = 12.5, linetype = "dashed",
                      color = rgb(50, 50, 50, max = 255)) +
           scale_x_discrete(breaks = c(-2125, 2125, 4125, 6125, 8125), 
                            labels = c("-2,000", "2,000", "4,000", "6,000", "8,000")) +
           scale_y_continuous(limits = c(0, 0.15)) 
  )
  
  data = data.table(read.csv(file.path(DATADIR,
                                       "npv_sim_density_20191103.csv")))
  
  data$prop_diff = 100 * data$prop_diff
  data$propbins = 100 * data$propbins
  data$midpoints = data$propbins + 2.5
  
  ggsave(filename = file.path(FIG_OUTDIR, "npv_sim_density.pdf"),
         width = 5, height = HEIGHT, plot =
           ggplot() +
           theme_few() +
           labs(x = "NPV Difference Relative to Carryback (%)", 
                y = "Density") +
           geom_bar(data = data, aes(x = reorder(as.character(midpoints), midpoints), 
                                   y = density), 
                    stat = "identity", fill = rgb(102, 102, 255, max = 255)) +
           geom_vline(data = data, aes(x = reorder(as.character(midpoints),
                                               midpoints)), xintercept = 20.5, linetype = "dashed",
                      color = rgb(50, 50, 50, max = 255)) +
           scale_x_discrete(breaks = c(-77.5, -52.5, -27.5, 27.5, 52.5, 77.5),
                            labels = c("-75", "-50", "-25", "25", "50", "75")) +
           scale_y_continuous(limits = c(0, 0.15)) 
  )
  
} # %>

# Figure 2c
graph_cf_claim <- function() { # %<
  
  data = data.table(read.csv(file.path(DATADIR,
                                       "lcf_claims_20191103.csv")))
  
  data = data[, list(cumulative_cfd = mean(cum_avg), cumulative_cfd2 = mean(cum_avg_max)), by = event_t]
  data$years = data$event_t

  ggsave(filename = file.path(FIG_OUTDIR, "cumulative_cfd_alt.pdf"),
         width = WIDTH, height = HEIGHT, plot =
           ggplot() +
           theme_few() +
           labs(x = "Years Since Initial Loss", 
                y = "Mean Cumulative Carryforwards/Initial Loss") +
           geom_point(data = data, aes(x = reorder(as.character(years), years), 
                                     y = cumulative_cfd, group = 1), 
                      stat = "identity", color = rgb(50, 50, 50, max = 255)) +
           geom_point(data = data, aes(x = reorder(as.character(years), years), 
                                     y = cumulative_cfd2, group = 2), 
                      stat = "identity", color = "gray48") +
           scale_y_continuous(limits = c(0, 1)) + 
           geom_line(data = data, aes(x = reorder(as.character(years), years), 
                                    y = cumulative_cfd, group = 1), 
                     stat = "identity", color = rgb(50, 50, 50, max = 255)) +
           geom_line(data = data, aes(x = reorder(as.character(years), years), 
                                    y = cumulative_cfd2, group = 2), 
                     stat = "identity", color = "gray48", linetype = "dashed") +
           theme(axis.text.y = element_text(color = "black")) + 
           theme(axis.text.x = element_text(color = "black")) + 
           theme(legend.position = "bottom", legend.title = element_blank())
  )
} # %>

# Figure 3
graph_npv <- function() { # %<
  
  data = data.table(read.csv(file.path(DATADIR,
                                       "size_claim_npv.csv.xls")))
  
  reshaped <- rbind(data[, list(X, v = unclaimed,
                                Group = "Unclaimed")],
                    data[, list(X, v = mistake,
                                Group = "Unclaimed and Positive NPV")])
  

    ggsave(filename = file.path(FIG_OUTDIR, "size_mistake_npv.pdf"),
           width = WIDTH, height = HEIGHT, plot =
             ggplot() +
             theme_few() +
             labs(x = "Size Percentile", 
                  y = "Share (Percent)") +
             geom_bar(data = reshaped, aes(x = reorder(as.character(X), X), y = v, fill = Group), 
                      position = "dodge",
                      stat = "identity") + 
             scale_fill_manual(values = c(rgb(102, 102, 255, max = 255), 
                                        rgb(204, 204, 255, max = 255))) +
             scale_x_discrete(labels = c("1" ="1-10", "2" = "11-20",
                                        "3" = "21-30", "4" = "31-40",
                                        "5" = "41-50", "6" = "51-60",
                                        "7" = "61-70", "8" = "71-80",
                                        "9" = "81-90", "10" = "91-92",
                                        "11" = "93-94", "12" = "95-96",
                                        "13" = "97-98", "14" = "99-100")) +
             theme(legend.position = "bottom", legend.title = element_blank()) +
             scale_y_continuous(limits = c(0,1)) + 
             theme(axis.text.x = element_text(angle = -20, hjust = 0.55, vjust = 0.35, color = "black")) + 
             theme(axis.text.y = element_text(color = "black"))
    )
    
    
  data = data.table(read.csv(file.path(DATADIR,
                                       "size_claim_all.csv.xls")))
  
  reshaped <- rbind(data[, list(X, v = unclaimed,
                                Group = "Unclaimed Refund > $1K")],
                    data[, list(X, v = unclaimed4,
                                Group = "Unclaimed Refund > $10K")],
                    # Focus on firms with >1M in sales
                    data[, list(X, v = ifelse(X > 4, unclaimed5, 0),
                                Group = "Unclaimed Refund > $100K")])
  reshaped$Group = factor(reshaped$Group, 
                          levels = c("Unclaimed Refund > $1K",
                                   "Unclaimed Refund > $10K",
                                   "Unclaimed Refund > $100K"))
  
  ggsave(filename = file.path(FIG_OUTDIR, "size_claim.pdf"),
         width=WIDTH, height=HEIGHT, plot=
           ggplot() +
           theme_few() +
           labs(x="Size Percentile", 
                y="Share (Percent)") +
           geom_bar(data=reshaped, aes(x=reorder(as.character(X), X), y=v,
                                       fill=Group), 
                    position="dodge",
                    stat="identity") + 
           scale_fill_manual(values=c(rgb(70, 70, 255, max=255), 
                                      rgb(153, 153, 255, max=255),
                                      rgb(204, 204, 255, max=255))) +
           scale_x_discrete(labels= c("1" ="1-10", "2" = "11-20",
                                      "3" = "21-30", "4" = "31-40",
                                      "5" = "41-50", "6" = "51-60",
                                      "7" = "61-70", "8" = "71-80",
                                      "9" = "81-90", "10" = "91-92",
                                      "11" = "93-94", "12" = "95-96",
                                      "13" = "97-98", "14" = "99-100")) +
           theme(legend.position="bottom", legend.title=element_blank()) +
           scale_y_continuous(limits=c(0,1)) +
           theme(axis.text.x = element_text(angle = -20, hjust = 0.55, vjust = 0.35, color="black")) + 
           theme(axis.text.y = element_text(color="black"))
  )
  
} # %>

# Figure 4, 5
graph_accountants <- function() { # %<
  
  data = data.table(read.csv(file.path(DATADIR,
                                       "cov.csv.xls")))
  
  ggsave(filename = file.path(FIG_OUTDIR, "covariance.pdf"),
         width = WIDTH, height = HEIGHT, plot =
           ggplot() +
           theme_few() +
           labs(x = "Years between Tax Returns",
                y = "Within-Firm Covariance of Claiming") +
           geom_pointrange(data = data, aes(x = reorder(as.character(event_t), event_t), 
                                          y = cov_same, ymin = lb_same, ymax = ub_same), 
                           stat = "identity", color = rgb(0, 0, 0, max = 255), 
                           fatten = 0.5, size = 0.4) +
           geom_pointrange(data = data, aes(x = reorder(as.character(event_t), event_t), 
                                          y = cov_diff, ymin = lb_diff, ymax = ub_diff), 
                           stat = "identity", color = rgb(102, 102, 255, max = 255),
                           fatten = 0.5, size = 0.4) +
           geom_line(data=data, aes(x = reorder(as.character(event_t), event_t),
                                    y = cov_diff, group = 1),
                     stat="identity", linetype = "dashed", color = rgb(102, 102, 255, max = 255)) +
           geom_line(data=data, aes(x = reorder(as.character(event_t), event_t),
                                    y = cov_same, group = 1),
                     stat = "identity", linetype = "dashed", color = rgb(0, 0, 0, max = 255)) +
           scale_y_continuous(limits = c(0,.1)) + 
           theme(axis.text.y = element_text(color = "black")) + 
           theme(axis.text.x = element_text(color = "black"))
  ) 
  
  data = data.table(read.csv(file.path(DATADIR,
                                       "prep_effects.csv")))
  
  ggsave(filename = file.path(FIG_OUTDIR, "prep_effects_alt.pdf"),
         width = WIDTH, height = HEIGHT, plot =
           ggplot() +
           theme_few() +
           labs(x = "Event Time Relative to Preparer Switch",
                y = "Coefficient on Change in Preparer Effect") +
           geom_pointrange(data = data[spec == 9], aes(x = reorder(as.character(time), time), 
                                                     y = beta, ymin = lb, ymax = ub), 
                           stat = "identity", color = rgb(0, 0, 0, max = 255)) +
           scale_y_continuous(limits = c(-1, 1.5)) + 
           geom_hline(data = data, aes(x = reorder(as.character(time), time), 
                                     y = beta), yintercept = 0, linetype = "dashed",
                      color = rgb(50, 50, 50,max = 255)) +
           theme(axis.text.y = element_text(color = "black")) + 
           theme(axis.text.x = element_text(color = "black"))
  ) 
  
} # %>


################################################################################ 
# Main
################################################################################ 
main <- function() {
  
  init()
  
  graph_losses()
  graph_npv_density()
  graph_cf_claim()
  graph_npv()
  graph_accountants()
  
}

main

