*******************************************************************************
*   Title: draft_accountants.do
* Purpose: Produces final tables for main draft, excluding appendix.
*          See the main() program at the bottom for organization.
*   Owner: Eric Zwick
*    Date: 2020-04-07
*******************************************************************************

* Figure 2a, 2b input
capture program drop fig_npv_robustness/*%<*/
program define fig_npv_robustness

    insheet using $csvdir/npv_sim.csv, clear
    gen diff = npv_cb_7r - npv_cf_7r
    gen prop_diff = diff/npv_cb_7r
    egen bins = cut(diff), at(-3000(250)9000)
    gen here = 1

    tempfile bins
    save `bins'

    collapse (sum) count=here (mean) diff, by(bins)
    sum count
    gen density = count/r(sum)

    list bins count diff density // this is npv.sim_20191103.csv

    * Proportional data
    use `bins', clear
    xtile propbin=prop_diff, n(20)
    egen propbins = cut(prop_diff), at(-1(.05)1)
    collapse (sum) count=here (mean) prop_diff, by(propbins)
    sum count
    gen density = count/r(sum)

    list propbins count prop_diff density // this is npv.sim_density_20191103.csv

end/*%>*/

* Figure 2c input
capture program drop fig_carryforward_xtras/*%<*/
program define fig_carryforward_xtras

    * INSIDE INSTRUCTIONS: see lcf_claims.csv in xp_20160306.R
    /* 
        comes from table nol_takeup_submission.data.balanced
        built by unbalanced.sql, 
            which tracks loss and nol claims for 1998-2002 firms, 
        then by balanced.py,
            which adds an indicator for fim exit

        draws from nol_npv_1998-2002.cf_claim.claiming_obs_cf and keeps
        those firms that have a nom_refund_obs = 0
    */

        * 1. Replicate graph in paper using lcf_cum.
        /**************************************************************************** 
            Code from xp_20160306.R 
            data = lcf_cum
            
            data$claim_avg = data$nol_claim / data$initial_loss
            data$cum_avg = data$cum_claim / data$initial_loss

            # Grab the earliest event_time for each NOL event where a firm no longer
            # has records
            fail_counts = data[firm_exit == 1, 
                               list(ein, event_t=min(event_t)), 
                               by=list(ein, nol_yr)][, list(failures=.N), 
                                                     keyby=event_t]
            firm_stats = data[, list(claim_avg=mean(claim_avg), 
                                     cum_avg=mean(cum_avg),
                                     count=.N), 
                                keyby=event_t]
            firm_stats = firm_stats[fail_counts]

            cum_claim defined == nol_cf - nol_remaining in nol_npv-1998-2002.unbalanced.sql
                where 
                   nol_cf == nol_initial from sim_refunds.py
                   nol_remaining is from claiming_obs_cf
                   nol_initial == -nol_amt from loss_event.txt
            
            event_t defined == tax_yr - nol_yr in unbalanced.sql as well

            initial_loss == nol_cf
        ****************************************************************************/
        tempfile lcf_short
        insheet using $csvdir/lcf_cum.csv, comma clear
        * vars: ein mean_payroll_13 mean_revenue_13 nol_yr event_t initial_loss
        * nol_claim cum_claim firm_exit nol_remaining
        * 3.2M obs, nols in 1998-2002
        gen claim_avg = nol_claim / initial_loss
        gen cum_avg = cum_claim / initial_loss
        tabstat claim_avg cum_avg, by(event_t) s(mean N)
        
        keep ein nol_yr event_t initial_loss nol_claim cum_claim claim_avg
        rename * *_lcf
        rename (ein_lcf nol_yr_lcf event_t_lcf) (ein nol_yr event_t)
        save `lcf_short'

        * 2. What do we get if we merge on claiming_obs_cf?
        import delimited $csvdir/claiming_obs_cf.csv, delim("|") clear
        rename (v1-v7) (ein nol_yr yr income_before_t nol_claim nol_remaining tax_saving)
        gen event_t = yr - nol_yr
        tab event_t

        merge 1:1 ein nol_yr event_t using `lcf_short', keep(2 3) 
        * Follow James in making this a balanced panel, where missings are zero
        gen claim_avg = nol_claim / initial_loss
        replace claim_avg = 0 if claim_avg == .
        tabstat claim_avg, by(event_t) s(mean N)

        * 3. What do we get if we merge on claiming_sim_cf?
        import delimited $csvdir/claiming_sim_cf.csv, delim("|") clear
        rename (v1-v7) (ein nol_yr yr income_before_t nol_claim nol_remaining tax_saving)
        gen event_t = yr - nol_yr
        tab event_t

        merge 1:1 ein nol_yr event_t using `lcf_short', keep(2 3) nogen
        gen claim_avg = nol_claim / initial_loss
        replace claim_avg = 0 if claim_avg == .
        tabstat claim_avg, by(event_t) s(mean N)

end/*%>*/

* Table 3
capture program drop table_firmsize_stats/*%<*/
program define table_firmsize_stats

    use $analysis/clients, clear
    make_size_bin 
    destring mean_payroll_13, force replace
    destring ebitda_13, force replace

    gen obs = 1
    bys ein: gen ein_count = _N
    bys ein: egen total_claims = total(iobs_refund)
    gen always_claim = cond(ein_count > 1 & ein_count < ., ///
                        cond(total_claims == ein_count, 1, 0), .)
    gen never_claim = cond(ein_count > 1 & ein_count < ., ///
                       cond(total_claims == 0, 1, 0), .)
    bys ein: replace ein_count = . if _n != 1

    gen external_prep = cond(!regexm(prep_ein, "X.*") | !regexm(prep_ssn, "Y.*"), 1, 0)

    #delimit ;
    tabstat 
        /* Chars */
        mean_revenue_13 mean_payroll_13 ebitda_13 
        /* Refund facts */
        external_prep
        sim_refund_13 iobs_refund 
        /* Propensity to claim */
        ein_count always_claim never_claim, 
        by(size_group) s(mean);
    #delimit cr

    bys ein: gen unique_firm = _n
    replace unique_firm = 0 if unique_firm != 1
    tabstat obs unique_firm sim_refund_13 obs_refund_13, by(size_group) s(sum)

end/*%>*/

* Table 4 
capture program drop table_switchers_firmsize/*%<*/
program define table_switchers_firmsize

    use $analysis/switchers, clear

    * Main table.
    #delimit ;
    
    xi: areg iobs_refund cpa atty other mis_profession i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, absorb(ein) cluster(ein); 

    xi: areg iobs_refund ltot_wages_13 mis_ltot_wages_13
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, absorb(ein) cluster(ein); 

    xi: areg iobs_refund self_emp_ind mis_self_emp_ind
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, absorb(ein) cluster(ein); 

    xi: areg iobs_refund age mis_age 
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, absorb(ein) cluster(ein); 

    xi: areg iobs_refund lmean_revenue_13 mis_lmean_revenue_13
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, absorb(ein) cluster(ein); 

    xi: areg iobs_refund 
            cpa atty other mis_profession
            ltot_wages_13 mis_ltot_wages_13
            self_emp_ind mis_self_emp_ind
            age mis_age
            lmean_revenue_13 mis_lmean_revenue_13
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, absorb(ein) cluster(ein); 

    #delimit cr

    * Regression for each firm size.
    make_size_bin

    forv i = 1/9 {

        #delimit ;
        xi: areg iobs_refund 
                cpa atty other 
                ltot_wages_13 
                self_emp_ind 
                age 
                lmean_revenue_13 
                mis_profession
                mis_ltot_wages_13
                mis_self_emp_ind
                mis_age
                mis_lmean_revenue_13
            i.nol_yr 
            lsim_refund_13 lrevenue_13 mis_lrevenue_13
            lassets_13 mis_lassets_13
            lebitda_13 mis_lebitda_13 if size_group == `i'
            , absorb(ein) cluster(ein)
            ; 
        test
                cpa atty other 
                ltot_wages_13 
                self_emp_ind 
                age 
                lmean_revenue_13;
        #delimit cr

    } 
    #delimit ;
    xi: areg iobs_refund 
                cpa atty other 
                ltot_wages_13 
                self_emp_ind 
                age 
                lmean_revenue_13 
                mis_profession
                mis_ltot_wages_13
                mis_self_emp_ind
                mis_age
                mis_lmean_revenue_13
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13 if size_group >= 10 & size_group <= 14
        , absorb(ein) cluster(ein)
        ; 
    test
            cpa atty other 
            ltot_wages_13 
            self_emp_ind 
            age 
            lmean_revenue_13;
    #delimit cr

    * Regression with deaths movers 
    use $analysis/deaths_movers, clear
    egen event_id = group(ein event_t_0)

    #delimit ;
    xi: areg iobs_refund cpa atty other mis_profession 
        i.nol_yr
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, absorb(event_id) cluster(ein); 

    xi: areg iobs_refund ltot_wages_13 mis_ltot_wages_13
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, absorb(event_id) cluster(ein);  

    xi: areg iobs_refund self_emp_ind mis_self_emp_ind
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, absorb(event_id) cluster(ein); 

    xi: areg iobs_refund age mis_age 
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, absorb(event_id) cluster(ein); 

    xi: areg iobs_refund lmean_revenue_13 mis_lmean_revenue_13
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, absorb(event_id) cluster(ein); 

    #delimit cr 

end/*%>*/

* Table 5
capture program drop table_switchers_iv/*%<*/
program define table_switchers_iv

    * From nol_takeup_levels/output/data.db in James directory
    tempfile prediction
    insheet using "$csvdir/prediction.csv", comma clear

    #delimit ;
    rename (v1-v11)
        (ein nol_yr prep_ssn_PRED mu_hat wage_hat self_hat age_hat mean_rev_hat
        tot_rev_hat churn_hat profession_hat);
    #delimit cr

    sum mu_hat, meanonly
    local avg_mu_hat = `r(mean)'

    save `prediction'

    use $analysis/deaths_movers, clear
    merge 1:1 ein nol_yr using `prediction', keep(3) nogen

    gen iv_mu_hat = cond(delta_time == -1, mu_hat, `avg_mu_hat')

    forv i = 1998/2011 {
        gen nol_yr_`i' = nol_yr == `i'
    }


    #delimit ;
    foreach v of varlist iobs_refund mu_hat iv_mu_hat lsim_refund_13 lrevenue_13
            mis_lrevenue_13 lassets_13 mis_lassets_13 lebitda_13 mis_lebitda_13 
            nol_yr_1998-nol_yr_2011 {;
       bys ein: egen mean_X = mean(`v');
       gen d`v' = `v' - mean_X;
       drop mean_X;
    };
    #delimit cr
    
    * 1. Spec 1
    ivreg2 diobs_refund (dmu_hat = div_mu_hat), first cluster(ein)

    * 2. Spec 2
    ivreg2 diobs_refund (dmu_hat = div_mu_hat) dnol_yr_*, first cluster(ein)

    * 3. Spec 3
    #delimit ;
    ivreg2 diobs_refund (dmu_hat = div_mu_hat) dnol_yr_*
        dlsim_refund_13 dlrevenue_13 dmis_lrevenue_13
        dlassets_13 dmis_lassets_13 dlebitda_13 dmis_lebitda_13, first cluster(ein);
    #delimit cr
    
    * Characteristics of the pre-event preparers
    keep if delta_time == -1
    sum age if mis_age == 0
    sum cpa atty other if mis_profession == 0
    sum ltot_wages if mis_ltot_wages == 0
    sum self_emp_ind if mis_self_emp == 0
    sum lmean_rev if mis_lmean_rev == 0

end/*%>*/

* Modified Figure 5
capture program drop fig_event_counting/*%<*/
program define fig_event_counting

    insheet using "$csvdir/switchers.csv", clear
    destring soi_db_ind, force replace
    keep if soi_db_ind == 1

    gen delta_time = event_t - event_t_0
    egen event_id = group(ein event_t_0)

    tempfile orig 
    save `orig'
    
    ***************************************************************************
    * Step 1: replicate graph using code in xp_20160306.R
    ***************************************************************************
    use `orig', clear
    keep if delta_time >= -2 & delta_time <= 1
    bys event_id: gen count = _N
    keep if count == 4

    bys ein: egen min_year = min(nol_yr_0)
    keep if nol_yr_0 == min_year
    gen delta_time_m1 = delta_time == -1
    gen delta_time_0 = delta_time == 0
    gen delta_time_p1 = delta_time == 1
    gen dmu_hat_0_x_m1 = dmu_hat_0 * delta_time_m1
    gen dmu_hat_0_x_0 = dmu_hat_0 * delta_time_0
    gen dmu_hat_0_x_p1 = dmu_hat_0 * delta_time_p1

    xi i.nol_yr

    areg iobs_refund delta_time_m1-delta_time_p1 dmu_hat_0_x_m1-dmu_hat_0_x_p1 ///
        _Inol_yr* lsim_refund_13 lrevenue_13 mis_lrevenue_13 ///
        lassets_13 mis_lassets_13 lebitda_13 mis_lebitda_13, absorb(ein) cluster(ein)
    
    ***************************************************************************
    * Step 2: add another event time period and fix the lag
    ***************************************************************************
    use `orig', clear
    keep if delta_time >= -3 & delta_time <= 2
    bys event_id: gen count = _N
    keep if count >= 4

    bys ein: egen min_year = min(nol_yr_0)
    keep if nol_yr_0 == min_year
    gen delta_time_m3 = delta_time == -3
    gen delta_time_m1 = delta_time == -1
    gen delta_time_0 = delta_time == 0
    gen delta_time_p1 = delta_time == 1
    gen delta_time_p2 = delta_time == 2
    gen dmu_hat_0_x_m3 = dmu_hat_0 * delta_time_m3
    gen dmu_hat_0_x_m1 = dmu_hat_0 * delta_time_m1
    gen dmu_hat_0_x_0 = dmu_hat_0 * delta_time_0
    gen dmu_hat_0_x_p1 = dmu_hat_0 * delta_time_p1
    gen dmu_hat_0_x_p2 = dmu_hat_0 * delta_time_p2

    xi i.nol_yr

    areg iobs_refund delta_time_m3-delta_time_p2 dmu_hat_0_x_m3-dmu_hat_0_x_p2 ///
        _Inol_yr* lsim_refund_13 lrevenue_13 mis_lrevenue_13 ///
        lassets_13 mis_lassets_13 lebitda_13 mis_lebitda_13, absorb(ein) cluster(ein)
    
    use `orig', clear
    keep if delta_time >= -3 & delta_time <= 1
    bys event_id: gen count = _N
    keep if count >= 4

    bys ein: egen min_year = min(nol_yr_0)
    keep if nol_yr_0 == min_year
    gen delta_time_m3 = delta_time == -3
    gen delta_time_m1 = delta_time == -1
    gen delta_time_0 = delta_time == 0
    gen delta_time_p1 = delta_time == 1
    gen dmu_hat_0_x_m3 = dmu_hat_0 * delta_time_m3
    gen dmu_hat_0_x_m1 = dmu_hat_0 * delta_time_m1
    gen dmu_hat_0_x_0 = dmu_hat_0 * delta_time_0
    gen dmu_hat_0_x_p1 = dmu_hat_0 * delta_time_p1

    xi i.nol_yr

    areg iobs_refund delta_time_m3-delta_time_p1 dmu_hat_0_x_m3-dmu_hat_0_x_p1 ///
        _Inol_yr* lsim_refund_13 lrevenue_13 mis_lrevenue_13 ///
        lassets_13 mis_lassets_13 lebitda_13 mis_lebitda_13, absorb(ein) cluster(ein)

    
end/*%>*/

* Extra facts
capture program drop bayes_preparers/*%<*/
program define bayes_preparers

    tempfile orig
    save `orig' 
    
    xi i.nol_yr
    foreach v of varlist sim_refund_13 revenue_13 assets_13 ebitda_13 {
        destring `v', force replace
        gen l`v' = log(`v')
        gen mis_l`v' = l`v' == .
        replace l`v' = -1 if l`v' == .
    }

    ***************************************************************************
    * Kane Staiger version
    ***************************************************************************
    * Model prep effect as 
    * (1) I_ijt = X_ijt * B + v_ijt where v_ijt = mu_j + theta_jt + e_ijt
    *   where mu_j is prep effect
    *   (prob safe for us to say theta_jt is zero, this is a peer FX term)
    *  
    * "The basic idea of the empirical Bayes approach is to multiply a noisy
    * estimate of teacher value added (e.g., the mean residual over all of a
    * teacher's students from a VA regression) by an estimate of its
    * reliability, where the reliability of a noisy estimate is the ratio of
    * signal variavne to signal plus noise variance."

    * 1. Estimate variance of mu_j (what john asked for)
    *    1. Estimate regression (1) above
        areg iobs_refund lsim_refund_13 _Inol_yr* lrevenue_13 mis_lrevenue_13 lassets_13 mis_lassets_13 lebitda_13 mis_lebitda_13, absorb(prep_id)

    *    2. Compute v_ijt = I_ijt - X_ijt * B
        #delimit ;
        gen iobs_refund_resid = iobs_refund - 
            _b[lsim_refund_13] * lsim_refund_13 - 
            _b[_Inol_yr_1999] * _Inol_yr_1999 - _b[_Inol_yr_2000] * _Inol_yr_2000 -
            _b[_Inol_yr_2001] * _Inol_yr_2001 - _b[_Inol_yr_2002] * _Inol_yr_2002 -
            _b[_Inol_yr_2003] * _Inol_yr_2003 - _b[_Inol_yr_2004] * _Inol_yr_2004 -
            _b[_Inol_yr_2005] * _Inol_yr_2005 - _b[_Inol_yr_2006] * _Inol_yr_2006 - 
            _b[_Inol_yr_2007] * _Inol_yr_2007 - _b[_Inol_yr_2008] * _Inol_yr_2008 -
            _b[_Inol_yr_2009] * _Inol_yr_2009 - _b[_Inol_yr_2010] * _Inol_yr_2010 - 
            _b[_Inol_yr_2011] * _Inol_yr_2011 - 
            _b[lrevenue_13] * lrevenue_13 - _b[lassets_13] * lassets_13 -
            _b[lebitda_13] * lebitda_13
            - _b[mis_lrevenue_13] * mis_lrevenue_13 
            - _b[mis_lassets_13] * mis_lassets_13 
            - _b[mis_lebitda_13] * mis_lebitda_13 
            ;
        #delimit cr

    *    3. Average v_ijt by j-t groups, keep total number of i for weights,
    *       drop duplicate preparer-firm obs to preserve independence assumption
    *       across e_ijt

        gen here = 1

        tempfile resid
        save `resid'
        use `resid', clear
        duplicates drop prep_id ein, force
        collapse (mean) iobs_refund_resid (sum) clients=here, by(prep_id nol_yr)

    *    4. Compute covariance of v^bar_jt and v^bar_jt-1, weighted by # of i
        sort prep_id nol_yr
        by prep_id: gen Liobs_refund_resid = iobs_refund_resid[_n-1]
        reg iobs_refund_resid Liobs_refund_resid [aw=clients]
        local beta = _b[Liobs_refund_resid]
        collapse (sd) sdx=Liobs_refund_resid [aw=clients]
        gen varx = sdx * sdx
        gen covar_prep = varx * `beta'
        list

        * No duplicates
        use `resid', clear
        collapse (mean) iobs_refund_resid (sum) clients=here, by(prep_id nol_yr)
        sort prep_id nol_yr
        by prep_id: gen Liobs_refund_resid = iobs_refund_resid[_n-1]
        reg iobs_refund_resid Liobs_refund_resid [aw=clients]
        local beta = _b[Liobs_refund_resid]
        collapse (sd) sdx=Liobs_refund_resid [aw=clients]
        gen varx = sdx * sdx
        gen covar_prep = varx * `beta'
        list

    * 2. Minimum variance unbiased estimate of mu_j (not necessary)

    use `orig', clear

end/*%>*/

*******************************************************************************
* Main
*******************************************************************************
capture program drop main
program define main

	capture log c
	cd $outputdir
	log using "draft_accountants.log", replace
	
	*** Figures ***	
	
	* Input to figures 2a and 2b
	fig_npv_robustness
	
	* Input to figure 2c
	fig_carryforward_xtras

    * Modified figure 5
    fig_event_counting
    
	*** Tables ***
	
	* Table 3
	table_firmsize_stats
	
	* Table 4 
	table_switchers_firmsize
	
	* Table 5
	table_switchers_iv

    * Facts
    bayes_preparers
	
    log close
end

main
